# File procdata/Src/t_listspec.x
# April 23, 2008
# By Doug Mink, SAO Telescope Data Center

include	<imhdr.h>
include	<smw.h>

define	LEN_USER_AREA	100000
define	SZ_FORMAT	8

# T_LISTSPEC - Compute and print the pixel value and corresponding wavelength
# for all the pixels in a list of one dimensional wavelength calibrated
# images.

procedure t_listspec()

int	list			# list of input images

char	image[SZ_FNAME]		# image file name
char	imname[SZ_FNAME]	# image name
char	outname[SZ_PATHNAME]	# Output file name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i, icol, ipix, ixpix
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
int	npts			# Number of pixels in spectrum
real	wavelength		# Wavelength at pixel in angstroms
double	wl			# Wavelength at pixel in angstroms
real	value			# Image value at pixel
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix, npix
int	mext			# FITS image extension
int	world			# wavelength or pixel

real	fpix, fmin, fmax, fnorm, fmean
int	outfd
double	dpix, wl1, wl2
 
bool	debug, verbose, heading, logwav, printlim
int	mode, nc, ns, ncol
double	fsum

real	clgetr()
int	clpopnu(), clgfil()
int	clgeti(), clscan()
bool	clgetb()
int	open()
int	strldx(), strncmp(), strlen()
double	wcs_p2w(), wcs_p2l()
real	renorm
bool	outfile
char	cdot, cslash
char	images[SZ_PATHNAME]
char	fluxform[SZ_FORMAT]
char	waveform[SZ_FORMAT]
char	pixform[SZ_FORMAT]
char	numform[SZ_FORMAT]
char	columns[8]
double	c0


begin
	cdot = '.'
	cslash = '/'
	c0 = 299792.5d0

	# Open input list of images
	list = clpopnu ("input")
	call clgstr ("input", images, SZ_PATHNAME)
	if (strlen (images) < 1) {
	    call printf ("LISTSPEC lists a spectrum in an arbitrary ASCII format\n")
	    call printf (" n=ap p=pixel w=wavelength f=flux v=velocity d=delta wavelength\n")
	    call printf (" can be listed in any order with specified formats.\n")
	    return
	    }

	mext = clgeti ("specext")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")
	world = 1

	call clgstr ("columns",columns,7)
	ncol = strlen (columns)
	printlim = clgetb ("printlim")

	minwav = clgetr ("lambda1")
	maxwav = clgetr ("lambda2")

	minpix = clgeti ("pix1")
	maxpix = clgeti ("pix2")

	verbose = clgetb ("verbose")
	

	debug = FALSE
	if (clscan ("debug") != EOF)
	    debug = clgetb ("debug")

	heading = FALSE
	if (clscan ("heading") != EOF)
	    heading = clgetb ("heading")

	renorm = 0.0
	if (clscan ("renormalize") != EOF)
	    renorm = clgetr ("renormalize")

	outfile = FALSE
	if (clscan ("outfile") != EOF)
	    outfile = clgetb ("outfile")

	call strcpy ("%g", fluxform, SZ_FORMAT)
	if (clscan ("fluxform") != EOF)
	    call clgstr ("fluxform", fluxform, SZ_FORMAT)

	logwav = FALSE
	if (clscan ("logwav") != EOF)
	    logwav = clgetb ("logwav")

	call strcpy ("%9.3f", waveform, SZ_FORMAT)
	if (clscan ("waveform") != EOF)
	    call clgstr ("waveform", waveform, SZ_FORMAT)

	call strcpy ("%3d", numform, SZ_FORMAT)
	if (clscan ("numform") != EOF)
	    call clgstr ("numform", numform, SZ_FORMAT)

	call strcpy ("%3d", pixform, SZ_FORMAT)
	if (clscan ("pixform") != EOF)
	    call clgstr ("pixform", pixform, SZ_FORMAT)

	mode = READ_ONLY

	if (debug) {
	    call printf ("LISTSPEC: parameters all read.\n")
	    call flush (STDOUT)
	    }

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    if (debug) {
		call printf ("Spectrum %s to be read\n")
		    call pargstr (image)
		call flush (STDOUT)
		}

	    # Try to open the image
	    call getimage (image, mext, mspec, mband, pix, im, sh, npts, imname,
			    mode, world, debug)
	    if (im == ERR)
		next

	    # Open output file
	    if (outfile) {
		nc = strldx (cdot, image)
		ns = strldx (cslash, image)
		if (nc > 0) {
		    if (strncmp (image[nc+1],"imh",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else if (strncmp (image[nc+1],"fit",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else
			call strcpy (image[ns+1], outname, 32)
		    }
		call strcat (".wav", outname, SZ_PATHNAME)
		outfd = open (outname, APPEND, TEXT_FILE)
		}
	    else
		outfd = STDOUT

	    if (heading) {
		if (mspec > 0 && mband > 0) {
		    call fprintf (outfd, "# %s[%d] %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (mband)
			call pargi (npts)
		    }
		else if (mspec > 0) {
		    call fprintf (outfd, "# %s[%d] %s : %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
	   	else if (mband > 0) {
		    call fprintf (outfd, "# %s %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
		else {
		    call fprintf (outfd, "# %s %s : %d pixels\n")
			call pargstr (image)
			call pargstr (imname)
			call pargi (npts)
		    }
		}

	# Get image wavelengths to list
	    wav = SX(sh)

	# Print image name
	    if (verbose) {
		if (outfile) {
		    call printf ("Spectrum %s %d - %d -> %s\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
			call pargstr (outname)
		    }
		else {
		    call printf ("Spectrum %s %d - %d\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
		    }
		call flush (STDOUT)
		}

	# Loop over pixels
	    if (minpix == INDEFI || minpix < NP1(sh))
		minpix = NP1(sh)
	    if (maxpix == INDEFI || maxpix > NP2(sh))
		maxpix = NP2(sh)

	    fsum = 0.d0
	    npix = 0
	    do i = minpix, maxpix {
		fpix = Memr[pix + i - 1]
		if (i == minpix) {
		    fmax = fpix
		    fmin = fpix
		    }
		else if (fpix > fmax)
		    fmax = fpix
		else if (fpix < fmin)
		    fmin = fpix
		fsum = fsum + double (fpix)
		npix = npix + 1
		}
	    fmean = fsum / double (npix)
	    if (debug) {
		call printf ("LISTSPEC: pixels %d-%d: %.8g, %.8g-%.8g\n")
		    call pargi (minpix)
		    call pargi (maxpix)
		    call pargr (fmean)
		    call pargr (fmin)
		    call pargr (fmax)
		}

	# Compute normalization factors so maximum value is renorm
	    if (renorm > 0.0) {
		fnorm = renorm / (fmax - fmin)
		if (debug) {
		    call printf ("LISTSPEC: fnorm = %.2f\n")
			call pargr (fnorm)
		    }
		}

	# Compute normalization factors so mean value is renorm
	    else if (renorm < 0.0) {
		fnorm = -renorm / fmean
		fmin = 0.0
		if (debug) {
		    call printf ("LISTSPEC: fnorm = %.2f\n")
			call pargr (fnorm)
		    }
		}
	    else {
		fnorm = 1.0
		fmin = 0.0
		}

	    if (debug) {
		call printf ("Output formats are w=%s, v=%s\n")
		    call pargstr (waveform)
		    call pargstr (fluxform)
		call flush (STDOUT)
		}

	    call wcs_set (sh)
	    if (printlim)
		ixpix = maxpix - minpix
	    else
		ixpix = 1
	    do ipix = minpix, maxpix, ixpix {
		dpix = double (ipix)
		wavelength = Memr[wav+i-1]
		if (logwav) {
		    wl = wcs_p2l (dpix)
		    wl1 = wcs_p2l (dpix-0.5d0)
		    wl2 = wcs_p2l (dpix+0.5d0)
		    }
		else {
		    wl = wcs_p2w (dpix)
		    wl1 = wcs_p2w (dpix-0.5d0)
		    wl2 = wcs_p2w (dpix+0.5d0)
		    }
		value = Memr[pix+ipix-1]
		if ((minwav == INDEFR || wavelength >= minwav) &&
		    (maxwav == INDEFR || wavelength <= maxwav)) {

		    if (renorm != 0.0)
			value = (value - fmin) * fnorm

		# Write values to the standard output
		    do icol = 1, ncol {
			if (columns[icol] == 'n') {
			    call fprintf (outfd, numform)
				call pargi (mspec)
			    }
			else if (columns[icol] == 'p') {
			    call fprintf (outfd, pixform)
				call pargi (ipix)
			    }
			else if (columns[icol] == 'w') {
			    call fprintf (outfd, waveform)
				call pargd (wl)
			    }
			else if (columns[icol] == 'f') {
			    call fprintf (outfd, fluxform)
				call pargr (value)
			    }
			else if (columns[icol] == 'd') {
			    call fprintf (outfd, waveform)
			    if (wl1 > wl2)
				call pargd (wl1 - wl2)
			    else
				call pargd (wl2 - wl1)
			    }
			else if (columns[icol] == 'v') {
			    call fprintf (outfd, waveform)
			    if (wl1 > wl2)
				call pargd (((wl1 - wl2) / wl) * c0)
			    else
				call pargd (((wl2 - wl1) / wl) * c0)
			    }
			if (icol < ncol)
			    call fprintf (outfd, " ")
			}
		    call fprintf (outfd, "\n")
		    }
		}

	# Flush standard output before opening next image
	    call flush (outfd)

	# Close image
	    call close_image (im, sh)
	    }

	# Close input list
	if (outfile)
	    call close (outfd)
	call clpcls (list)
	return
end

# Mar 25 2008	New task based on listwav
# Apr 23 2008	Fix delta wavelength bug
